package com.pij.lottieshow.list;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.TextView;

import com.pij.lottieshow.R;
import com.pij.lottieshow.detail.LottieActivity;
import com.pij.lottieshow.detail.LottieFragment;
import com.pij.lottieshow.model.Converter;
import com.pij.lottieshow.model.LottieContent;
import com.pij.lottieshow.model.LottieFile;
import com.pij.lottieshow.model.LottieUi;
import com.pij.lottieshow.saf.SafClient;
import com.pij.lottieshow.ui.LibraryString;
import com.pij.lottieshow.ui.Utils;

import org.apache.commons.collections4.IterableUtils;
import org.apache.commons.lang3.BooleanUtils;
import org.apache.commons.lang3.tuple.Pair;

import javax.inject.Inject;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.Unbinder;
import dagger.android.support.DaggerAppCompatActivity;
import rx.Observable;
import rx.Single;
import rx.Subscription;
import rx.functions.Actions;
import rx.functions.Func1;
import rx.subjects.BehaviorSubject;
import rx.subscriptions.CompositeSubscription;

import static com.jakewharton.rxbinding.view.RxView.clicks;
import static rx.Observable.empty;
import static rx.Observable.from;
import static rx.Observable.just;

/**
 * An activity representing a list of Lotties. This activity
 * has different presentations for handset and tablet-size devices. On
 * handsets, the activity presents a list of items, which when touched,
 * lead to a {@link LottieActivity} representing
 * item label. On tablets, the activity presents the list of items and
 * item label side-by-side using two vertical panes.
 */
public class LottiesActivity extends DaggerAppCompatActivity {

    private static final int REQUEST_CODE_PICK = 24;

    private final CompositeSubscription subscriptions = new CompositeSubscription();
    private final BehaviorSubject<Boolean> onCreate = BehaviorSubject.create();
    @Nullable
    @BindView(R.id.lottie_detail_container)
    View detailContainer;
    @BindView(R.id.toolbar)
    Toolbar toolbar;
    @BindView(R.id.fab)
    FloatingActionButton fab;
    @BindView(R.id.lottie_list)
    RecyclerView list;
    @BindView(R.id.empty)
    TextView empty;
    @Inject
    LottiesViewModel viewModel;
    @Inject
    SafClient saf;
    @Inject
    Converter converter;
    @Inject
    LibraryString libraryString;
    private Unbinder unbinder;
    private Snackbar progress;

    @NonNull
    public static Intent createIntent(Context context) {
        return new Intent(context, LottiesActivity.class);
    }

    public LottiesActivity() {}

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_lottie_list);
        unbinder = ButterKnife.bind(this);

        setSupportActionBar(toolbar);

        progress = Snackbar.make(fab, "Loading...", Snackbar.LENGTH_INDEFINITE);

        LottiesAdapter adapter = new LottiesAdapter(R.layout.lottie_list_item);
        list.setAdapter(adapter);

        subscriptions.addAll(clicks(fab).subscribe(click -> pickJsonFile(), this::notifyError),
                             saf.analysed().subscribe(viewModel::addLottie, this::notifyError),
                             saf.inProgress().subscribe(this::showProgress, this::notifyError),

                             adapter.itemClicked().flatMapSingle(this::toModel)
                                    .subscribe(viewModel::select, this::notifyError),

                             // error handling
                             viewModel.shouldShowList().subscribe(ignored -> Actions.empty(), this::notifyError),
                             viewModel.shouldShowList()
                                      .flatMap(list -> from(list).flatMapSingle(this::fromModel).toList())
                                      .subscribe(adapter::setItems, e -> empty()),

                             // The detail container view will be present only in the
                             // large-screen layouts (res/values-w900dp).
                             // If this view is present, then the
                             // activity should be in two-pane mode.
                             detailContainer == null
                             ? showInActivity(viewModel.shouldShowLottie())
                             : showInFragment(viewModel.shouldShowLottie()),

                             viewModel.shouldShowList()
                                      .map(IterableUtils::isEmpty)
                                      .map(empty -> empty ? View.VISIBLE : View.GONE)
                                      .subscribe(empty::setVisibility, e -> empty()),
                             Observable.zip(viewModel.shouldShowList().map(IterableUtils::isEmpty),
                                            onCreate,
                                            (emptyList, firstCreate) -> emptyList && firstCreate)
                                       .filter(BooleanUtils::isTrue)
                                       .take(1)
                                       .subscribe(this::animateFab, e -> empty())

        );

        onCreate.onNext(savedInstanceState == null);
        System.out.println("PJC in onCreate");
    }

    @Override
    protected void onDestroy() {
        fab.clearAnimation();
        subscriptions.clear();
        unbinder.unbind();
        super.onDestroy();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        switch (requestCode) {
            case REQUEST_CODE_PICK:
                switch (resultCode) {
                    case RESULT_OK:
                        saf.analyse(data);
                        break;
                    default:
                        break;
                }
                break;
            default:
                super.onActivityResult(requestCode, resultCode, data);
        }
    }

    /**
     * Put the library version in the menu.
     * We don't do anything when the _only_ menu is clicked.
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.lotties_menu, menu);
        libraryString.configure(menu);
        return true;
    }

    private void animateFab(boolean extraordinary) {
        int resource = extraordinary ? R.anim.pulses : R.anim.slide_in_right;
        Animation animation = AnimationUtils.loadAnimation(this, resource);
        fab.startAnimation(animation);
    }

    private Single<LottieFile> toModel(LottieUi ui) {
        return converter.toModel(ui);
    }

    private Single<Pair<LottieUi, LottieContent>> fromModel(LottieFile model) {
        return Single.zip(converter.fromModel(model), Single.just(model.getContent()).map(LottieContent::create),
                          Pair::of);
    }

    private void showProgress(boolean inProgress) {
        if (inProgress) { progress.show(); } else progress.dismiss();
    }

    private void notifyError(Throwable error) {
        Utils.notifyError(error, fab);
    }

    private void pickJsonFile() {
        saf.pickJsonFile(this, REQUEST_CODE_PICK);
    }

    private Subscription showInFragment(Observable<LottieFile> lottieFile) {
        return lottieFile.compose(mapWithoutError(LottieFragment::createInstance))
                         .subscribe(this::setDetailFragment, this::notifyError);
    }

    private Subscription showInActivity(Observable<LottieFile> lottieFile) {
        return lottieFile.compose(mapWithoutError(i -> LottieActivity.createIntent(this, i)))
                         .subscribe(this::startActivity, this::notifyError);
    }

    @NonNull
    private <T> Observable.Transformer<LottieFile, T> mapWithoutError(final Func1<LottieUi, T> mapper) {
        return lottie -> lottie.flatMapSingle(this::fromModel)
                               .map(Pair::getLeft)
                               .flatMap(item -> just(item).map(mapper)
                                                          .doOnError(this::notifyError)
                                                          .onErrorResumeNext(empty()));
    }

    private int setDetailFragment(LottieFragment fragment) {
        return getSupportFragmentManager().beginTransaction().replace(R.id.lottie_detail_container, fragment).commit();
    }
}
